/***************************************************************************
                          krecfilewidgets.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Arnold Krille
    email                : arnold@arnoldarts.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License.               *
 *                                                                         *
 ***************************************************************************/

#include "krecfilewidgets.h"
#include "krecfilewidgets.moc"

#include "krecfile.h"
#include "krecglobal.h"

#include <kdebug.h>
#include <klocale.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qpointarray.h>
#include <qlineedit.h>

#include <qregion.h>
#include <qrect.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <kpopupmenu.h>
#include <kactionclasses.h>
#include <kaction.h>

KRecFileWidget::KRecFileWidget( KRecFile* file, QWidget* p, const char* n )
  : QFrame( p,n )
  , _file( 0 )
{
kdDebug( 60005 ) << k_funcinfo << file << endl;
	this->setFrameStyle( QFrame::Panel|QFrame::Sunken );
	this->setLineWidth( 1 );
	this->setMinimumHeight( 20 );

	setFile( file );
}
KRecFileWidget::~KRecFileWidget() {
kdDebug( 60005 ) << k_funcinfo << endl;
}

void KRecFileWidget::setFile( KRecFile* file ) {
kdDebug( 60005 ) << k_funcinfo << file << endl;
	if ( _file != file ) {
		_file = file;
			//kdDebug( 60005 ) << "Removing widgets" << endl;
			QValueList<KRecBufferWidget*>::iterator it = bufferwidgets.begin();
			while ( it != bufferwidgets.end() ) {
				delete ( *it );
				++it;
			}
			bufferwidgets.clear();
			resizeEvent();
		if ( _file ) {
			for ( QValueList<KRecBuffer*>::iterator it = _file->_buffers.begin(); it != _file->_buffers.end(); ++it ) {
				newBuffer( ( *it ) );
			}
			connect( _file, SIGNAL( sNewBuffer( KRecBuffer* ) ), this, SLOT( newBuffer( KRecBuffer* ) ) );
			connect( _file, SIGNAL( sDeleteBuffer( KRecBuffer* ) ), this, SLOT( deleteBuffer( KRecBuffer* ) ) );
			//kdDebug( 60005 ) << _file->_currentBuffer << endl;
		}
	}
}

void KRecFileWidget::resizeEvent( QResizeEvent* /*qre*/ ) {
	//kdDebug( 60005 ) << k_funcinfo << endl;
	if ( _file ) {
		int w = contentsRect().width();
		int h = contentsRect().height();
		QValueList<KRecBufferWidget*>::iterator it;
		for ( it = bufferwidgets.begin(); it != bufferwidgets.end(); ++it ) {
			int width, x;
			if ( !_file->offsetSize()==0 && !( *it )->buffer()->size()==0 ) {
				width = int ( float( ( *it )->buffer()->size() ) / _file->offsetSize() * w );
				x = int( float( ( *it )->buffer()->startpos() ) / _file->size() * w ) + contentsRect().left() ;
			} else {
				width = 5;
				x = contentsRect().left();
			}
			( *it )->setGeometry( x, contentsRect().top(), width, h );
		}
	}
}

void KRecFileWidget::mouseReleaseEvent( QMouseEvent* qme ) {
kdDebug( 60005 ) << k_funcinfo << "(" << qme->x() << "|" << qme->y() << ")" << endl;
}

void KRecFileWidget::newBuffer( KRecBuffer* buffer ) {
kdDebug( 60005 ) << k_funcinfo << buffer << endl;
	KRecBufferWidget *tmp = new KRecBufferWidget( buffer, this );
	bufferwidgets.append( tmp );
	tmp->show();
	resizeEvent();
}
void KRecFileWidget::deleteBuffer( KRecBuffer* buffer ) {
kdDebug( 60005 ) << k_funcinfo << buffer << endl;
kdDebug( 60005 ) << bufferwidgets.count() << endl;
	QValueList<KRecBufferWidget*>::iterator it = bufferwidgets.begin();
	KRecBufferWidget* tmp = 0;
	while ( it != bufferwidgets.end() ) {
		if ( ( *it )->buffer() == buffer ) tmp = ( *it );
		++it;
	}
	if ( tmp != 0 ) {
		delete tmp;
		bufferwidgets.remove( tmp );
	}
kdDebug( 60005 ) << bufferwidgets.count() << endl;
}

KRecBufferWidget::KRecBufferWidget( KRecBuffer* buffer, QWidget* p, const char* n )
  : QFrame( p,n )
  , _buffer( buffer )
  , _main_region( 0 ), _title_region( 0 ), _fileend_region( 0 )
{
	_activeaction = new KToggleAction( i18n( "Toggle Active/Disabled State" ), KShortcut(), this );
	_activeaction->setChecked( _buffer->active() );
	connect( _activeaction, SIGNAL( toggled( bool ) ), _buffer, SLOT( setActive( bool ) ) );
	connect( _buffer, SIGNAL( activeChanged( bool ) ), _activeaction, SLOT( setChecked( bool ) ) );

	_removeaction = new KAction( i18n( "Remove this part" ), "fileremove", KShortcut(), _buffer, SLOT( deleteBuffer() ), this );

	connect( _buffer, SIGNAL( somethingChanged() ), this, SLOT( update() ) );
//	this->setFrameStyle( QFrame::Panel|QFrame::Raised );
//	this->setLineWidth( 1 );
//	this->setPaletteBackgroundColor( QColor( 255,125,125 ) );
kdDebug( 60005 ) << k_funcinfo << endl;
}
KRecBufferWidget::~KRecBufferWidget() {
kdDebug( 60005 ) << k_funcinfo << endl;
}

void KRecBufferWidget::resizeEvent( QResizeEvent* ) {
	//kdDebug( 60005 ) << k_funcinfo << endl;
	_title_height = this->fontMetrics().boundingRect( _buffer->filename() ).height() + 4;
	if( this->height()/5 > _title_height ) _title_height = this->height()/5;
	int _title_width = this->fontMetrics().boundingRect( _buffer->filename() ).width() + 10;
	if ( _title_width > this->width() ) _title_width = this->width();
	if ( _main_region ) delete _main_region;
	_main_region = new QRegion( QRect( 0, _title_height, this->width(), this->height()-_title_height ) );
	if ( _title_region ) delete _title_region;
	_title_region = new QRegion( QRect( 0, 0, _title_width, _title_height ) );
	if ( _fileend_region ) { delete _fileend_region; _fileend_region=0; }
	_fileend_region = new QRegion( QRect( this->width()-4, _title_height/2, 4, _title_height/2 ) );

	this->setMask( _main_region->unite( *_title_region ).unite( *_fileend_region ) );
	
	_topleft = _title_region->boundingRect().topLeft();
	_bottomleft = _main_region->boundingRect().bottomLeft();
	_bottomright = _main_region->boundingRect().bottomRight();
	_topright = _main_region->boundingRect().topRight();
	_bottommiddle = _title_region->boundingRect().bottomRight();
	_bottommiddle += QPoint( 0, 1 );
	_topmiddle = _title_region->boundingRect().topRight();
}
void KRecBufferWidget::drawFrame( QPainter* p ) {
	//kdDebug( 60005 ) << k_funcinfo << endl;
	if ( _buffer->active() )
		p->setBrush( this->colorGroup().highlight() );
	else
		p->setBrush( this->colorGroup().highlight().dark() );
	p->setPen( QPen( this->colorGroup().dark(), 1 ) );
	p->drawRect( _title_region->boundingRect() );
	p->drawRect( _fileend_region->boundingRect() );

	p->setBrush( QBrush() );
	p->setPen( QPen( this->colorGroup().dark(), 1 ) );
	p->drawRect( _main_region->boundingRect() );

	p->setPen( QPen( QColor( 0,0,0 ) ) );
	p->drawText( _title_region->boundingRect(), Qt::AlignCenter, _buffer->filename() );

/*	p->setPen( QPen( this->colorGroup().dark(), 1 ) );
	p->drawLine( _bottomleft, _bottomright );
	p->drawLine( _bottomright, _topright );
	p->drawLine( _topmiddle, _bottommiddle );
	p->setPen( QPen( this->colorGroup().light(), 1 ) );
	p->drawLine( _bottomleft, _topleft );
	p->drawLine( _topleft, _topmiddle );
	p->drawLine( _bottommiddle, _topright );*/
}
void KRecBufferWidget::drawContents( QPainter* p ) {
	//kdDebug( 60005 ) << k_funcinfo << endl;
	initSamples();
	int space = _main_region->boundingRect().height()/2;
	int xoffset = _main_region->boundingRect().top() + space;
	p->setPen( QPen( QColor( 255,0,0 ) ) );
	for ( uint i=0; i<samples1.count(); i++ )
		p->drawPoint( i, samples1[ i ]->getMax()/float( samples2[ 0 ]->getMax() ) *space +xoffset );
	p->setPen( QPen( QColor( 0,0,255 ) ) );
	for ( uint i=0; i<samples1.count(); i++ )
		p->drawPoint( i, samples1[ i ]->getMin()/float( samples2[ 0 ]->getMin() ) *space +xoffset );
	p->setPen( QPen( QColor( 0,0,0 ) ) );
	p->drawText( _main_region->boundingRect(), Qt::AlignCenter, "Lots of Data" );
}
void KRecBufferWidget::paintEvent( QPaintEvent* ) {
	QPainter *p = new QPainter( this );
	drawFrame( p );
	drawContents( p );
	delete p;
}

void KRecBufferWidget::mousePressEvent( QMouseEvent* qme ) {
	if ( _main_region->contains( qme->pos() ) || _title_region->contains( qme->pos() ) ) {
		if ( qme->button() == Qt::RightButton ) {
			KPopupMenu tmp( this );
			_activeaction->plug( &tmp );
			_removeaction->plug( &tmp );
			tmp.exec( qme->globalPos() );
		}
	}
}

void KRecBufferWidget::initSamples() {
kdDebug( 60005 ) << k_funcinfo << endl;
/*	samples1.setAutoDelete( true );
	samples1.clear();
	samples2.setAutoDelete( true );
	samples2.clear();
	Sample* tmp = new Sample();
	samples1.append( tmp );
	int j=0;
	//for ( int i=0; i<= _buffer->sizeInSamples(); ++i ) {
	for ( int i=0; i<= 10000; ++i ) {
		if ( j/128 ) {
//kdDebug() << "sample " << i << ": creating new Sample" << endl;
			tmp = new Sample();
			samples1.append( tmp );
			j=0;
		}
		tmp->addValue( _buffer->getSample( i, 0 ) );
		++j;
	}

	tmp = new Sample;
	samples2.append( tmp );
	j=0;
	for ( uint i=0; i<samples1.count(); ++i ) {
		if ( j/128 ) {
			tmp = new Sample();
			samples2.append( tmp );
			j=0;
		}
		tmp->addValue( samples1.at( i )->getValue() );
		++j;
	}
*/
/*	for ( uint i=0; i<samples1.count(); ++i )
		kdDebug() << i << ": " << samples1.at( i )->getMin() << " < " << samples1.at( i )->getValue() << " < " << samples1.at( i )->getMax() << endl;
	for ( uint i=0; i<samples2.count(); ++i )
		kdDebug() << i << ": " << samples2.at( i )->getMin() << " < " << samples2.at( i )->getValue() << " < " << samples2.at( i )->getMax() << endl;
*/
}

